#pragma once


//-----------------------------------------------------------------------------
//
//  Type: GPIO_PORT
//
//  Specifies the GPIO Port
//
//-----------------------------------------------------------------------------
typedef enum
{
    GPIO_PORT_1,
    GPIO_PORT_2,
    GPIO_PORT_3,
    GPIO_PORT_4
} GPIO_PORT;

//-----------------------------------------------------------------------------
//
//  Type: GPIO_PIN
//
//  Specifies the GPIO Pin
//
//-----------------------------------------------------------------------------
typedef enum
{
    GPIO_PIN_0,
    GPIO_PIN_1,
    GPIO_PIN_2,
    GPIO_PIN_3,
    GPIO_PIN_4,
    GPIO_PIN_5,
    GPIO_PIN_6,
    GPIO_PIN_7,
    GPIO_PIN_8,
    GPIO_PIN_9,
    GPIO_PIN_10,
    GPIO_PIN_11,
    GPIO_PIN_12,
    GPIO_PIN_13,
    GPIO_PIN_14,
    GPIO_PIN_15,
    GPIO_PIN_16,
    GPIO_PIN_17,
    GPIO_PIN_18,
    GPIO_PIN_19,
    GPIO_PIN_20,
    GPIO_PIN_21,
    GPIO_PIN_22,
    GPIO_PIN_23,
    GPIO_PIN_24,
    GPIO_PIN_25,
    GPIO_PIN_26,
    GPIO_PIN_27,
    GPIO_PIN_28,
    GPIO_PIN_29,
    GPIO_PIN_30,
    GPIO_PIN_31
} GPIO_PIN;


//-----------------------------------------------------------------------------
//
//  Type: GPIO_FUNCTION
//
//  Specifies the FUNCTION for a GPIO.
//
//-----------------------------------------------------------------------------
typedef enum 
{
    GPIO_FUNCTION_ALT0,
    GPIO_FUNCTION_ALT1,
    GPIO_FUNCTION_ALT2,
    GPIO_FUNCTION_ALT3,
    GPIO_FUNCTION_ALT4,
    GPIO_FUNCTION_ALT5,
    GPIO_FUNCTION_ALT6,
    GPIO_FUNCTION_ALT7
} GPIO_FUNCTION;


//-----------------------------------------------------------------------------
//
//  Type: GPIO_DIR
//
//  Specifies the direction of the GPIO.
//
//-----------------------------------------------------------------------------
typedef enum 
{
    GPIO_DIR_IN,
    GPIO_DIR_OUT
} GPIO_DIR;


//-----------------------------------------------------------------------------
//
//  Type: GPIO_INTR
//
//  Specifies the detection logic used for GPIO interrupts.
//
//-----------------------------------------------------------------------------
typedef enum 
{
    GPIO_INTR_LOW_LEV,
    GPIO_INTR_HIGH_LEV,
    GPIO_INTR_RISE_EDGE,
    GPIO_INTR_FALL_EDGE,
    GPIO_INTR_BOTH_EDGE,
    GPIO_INTR_NONE
} GPIO_INTR;


//-----------------------------------------------------------------------------
//
//  Type: GPIO_LOOPBACK
//
//  Specifies the loopback mode for a GPIO
//
//-----------------------------------------------------------------------------
typedef enum
{
    GPIO_LOOPBACK_DISABLE,
    GPIO_LOOPBACK_ENABLE,
    GPIO_LOOPBACK_INVALID
} GPIO_LOOPBACK;


//-----------------------------------------------------------------------------
//
//  Type: GPIO_SLEW
//
//  Specifies the slew rate for a GPIO
//
//-----------------------------------------------------------------------------
typedef enum
{
    GPIO_SLEW_SLOW,
    GPIO_SLEW_FAST,
    GPIO_SLEW_INVALID
} GPIO_SLEW;

//-----------------------------------------------------------------------------
//
//  Type: GPIO_DRIVE
//
//  Specifies the drive strength for a GPIO.
//
//-----------------------------------------------------------------------------
typedef enum 
{
    GPIO_DRIVE_NORMAL,
    GPIO_DRIVE_HIGH,
    GPIO_DRIVE_MAX,
    GPIO_DRIVE_INVALID
} GPIO_DRIVE;

//-----------------------------------------------------------------------------
//
//  Type: GPIO_OPENDRAIN
//
//  Specifies the open drain for a GPIO.
//
//-----------------------------------------------------------------------------
typedef enum 
{
    GPIO_OPENDRAIN_DISABLE,
    GPIO_OPENDRAIN_ENABLE,
    GPIO_OPENDRAIN_INVALID
} GPIO_OPENDRAIN;

//-----------------------------------------------------------------------------
//
//  Type: GPIO_PULL
//
//  Specifies the pull-up/pull-down/keeper configuration for a GPIO.
//
//-----------------------------------------------------------------------------
typedef enum 
{
    GPIO_PULL_NONE,
    GPIO_PULL_KEEPER,
    GPIO_PULL_DOWN_100K,
    GPIO_PULL_UP_47K,
    GPIO_PULL_UP_100K,
    GPIO_PULL_UP_22K,
    GPIO_PULL_INVALID
} GPIO_PULL;

//-----------------------------------------------------------------------------
//
//  Type: GPIO_HYSTERESIS
//
//  Specifies the hysteresis for a pad.
//
//-----------------------------------------------------------------------------
typedef enum 
{
    GPIO_HYSTERESIS_DISABLE,
    GPIO_HYSTERESIS_ENABLE,
    GPIO_HYSTERESIS_INVALID,
} GPIO_HYSTERESIS;

//-----------------------------------------------------------------------------
//
//  Type: GPIO_VOLTAGE
//
//  Specifies the driver voltage for a pad.
//
//-----------------------------------------------------------------------------
typedef enum 
{
    GPIO_VOLTAGE_3V3,
    GPIO_VOLTAGE_1V8,
    GPIO_VOLTAGE_INVALID
} GPIO_VOLTAGE;


typedef struct
{
    GPIO_PORT       Port;       // GPIO Port
    GPIO_PIN        Pin;        // GPIO Pin
    GPIO_FUNCTION   Function;   // GPIO Function - Only used for GET_CONFIG
    GPIO_DIR        Direction;  // GPIO Direction
    GPIO_INTR       Interrupt;  // GPIO Interrupt Configuration
    GPIO_LOOPBACK   Loopback;   // GPIO Loopback mode
    GPIO_SLEW       Slew;       // GPIO Slew rate
    GPIO_DRIVE      Drive;      // GPIO Drive Strength
    GPIO_OPENDRAIN  OpenDrain;  // GPIO Open Drain mode
    GPIO_PULL       Pull;       // GPIO Pull up/pull down/keep Configuration
    GPIO_HYSTERESIS Hysteresis; // GPIO Hysteresis mode
    GPIO_VOLTAGE    Voltage;    // GPIO Voltage
} GPIO_CONFIG, *LPGPIO_CONFIG;

typedef struct
{
    GPIO_PORT       Port;       // GPIO Port
    union                       // GPIO Pin -or- Mask
    {
        GPIO_PIN    Pin;
        UINT32      Mask;
    };
    UINT32          Value;      // GPIO Value
} GPIO, *LPGPIO;
